
/**************************************************************************************
**
**  The Osiris application compares one catalog of executable files
**  with another catalog of executable files. Osiris catalogs specified
**  directories of files (including MD5 hashes, modification dates,
**  and file attributes into a specified database.
**
**  Copyright (C) 2000  - the shmoo group -
**
**  This program is free software; you can redistribute it and/or
**  modify it under the terms of the GNU General Public License
**  as published by the Free Software Foundation; either version 2
**  of the License, or (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
** 
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
**
**************************************************************************************/

/**********************************************************************
**
**    The Shmoo Group (TSG)
**
**    File:      hash_handler.c
**    Author:    Brian Wotring
**
**    Project:   osiris ( UNIX ).
**
**********************************************************************/


#include "hash_handler.h"


char * hash_file_haval( const char *file_path )
{
    unsigned char fingerprint[FPTLEN >> 3];
    char *file_hash;

    int index;

    if( haval_file( (char *)file_path, fingerprint ) )
	return NULL;

    file_hash = safemalloc( 65 );

    for( index = 0; index < FPTLEN >> 3; index++ )
        sprintf( &file_hash[index * 2], "%02X", fingerprint[index] );

    file_hash[64] = '\0';

    return file_hash;
}

char * hash_file_md5( const char *file_path )
{
    FILE *file_to_hash	= NULL;
    MD5_CTX context;
    unsigned char digest[MD5_DIGEST_LENGTH];
    
    int burst;
    int index;

    static unsigned char buffer[BUFFER_SIZE];
    char *file_hash;
    
    if( ( file_to_hash = fopen( file_path, "r" ) ) == NULL )
	return NULL;

    /* initialize the md5 library and our hash string */
    /* which is 32 bytes long for md5.                */
    MD5_Init( &context );
    
    file_hash = safemalloc( 33 );

    while(1)
    {
        burst = read( fileno( file_to_hash ), buffer, BUFFER_SIZE );

        if( burst <= 0 )
	    break;

        MD5_Update( &context, buffer, (unsigned long)burst );
    }

    MD5_Final( &( digest[0] ), &context );

    for ( index = 0; index < MD5_DIGEST_LENGTH; index++)
        sprintf( &file_hash[index * 2], "%02x", digest[index] );

    file_hash[32] = '\0';
    fclose( file_to_hash );

    return file_hash;
}

char * hash_file_sha( const char *file_path )
{
    FILE *file_to_hash  = NULL;
    SHA_CTX context;
    unsigned char digest[SHA_DIGEST_LENGTH];

    int burst;
    int index;

    static unsigned char buffer[BUFFER_SIZE];
    char *file_hash;

    if( ( file_to_hash = fopen( file_path, "r" ) ) == NULL )
        return NULL;

    /* initialize the sha library and our hash string */
    /* which is 40 bytes long for sha.                */

    SHA_Init( &context );
    file_hash = safemalloc( 41 );

    while(1)
    {
        burst = read( fileno( file_to_hash ), buffer, BUFFER_SIZE );


        if( burst <= 0 )
            break;


        SHA_Update( &context, buffer, (unsigned long)burst );
    }

    SHA_Final( &( digest[0] ), &context );

    for ( index = 0; index < SHA_DIGEST_LENGTH; index++)
        sprintf( &file_hash[index * 2], "%02x", digest[index] );

    file_hash[40] = '\0';
    fclose( file_to_hash );

    return file_hash;
}
